import greenfoot.*;
import java.awt.Color;
import java.util.ArrayList;

/**
 * The world in which the rings are drawn
 * 
 * @author Neil Thistlethwaite
 * @version 1.0
 */

public class TowerOfHanoi extends World
{
    int numrings = 5;
    int start = 1;
    int finish = 3;
    int oinstructionSubStep = 1;
    int instructionSubStep = 1;
    int currentInstruction = 0;
    Instruction icurrentInstruction = null;
    Counter stepCounter = new Counter("Steps: ");
    boolean finishedInstruction = true;
    Stack stack1 = new Stack(1, 150);
    Stack stack2 = new Stack(2, 300);
    Stack stack3 = new Stack(3, 450);
    Ring moving;
    ArrayList<Ring> rings;
    ArrayList<Color> colors = new ArrayList<Color>(21);
    ArrayList<Instruction> instructions;
    
    public TowerOfHanoi() {
        super(600, 400, 1);
        int numOfRings = InputWorld.rings;
        numrings = numOfRings;
        rings = new ArrayList<Ring>(numrings+1);
        instructions = new ArrayList<Instruction>((int)(Math.pow(2.0, (numrings*1.0)))+2);
        GreenfootImage myImage = new GreenfootImage(600, 400);
        myImage.setColor(new Color(150, 150, 150));
        myImage.fill();
        myImage.setColor(new Color(230, 230, 230));
        myImage.fillRect(50, 50, 500, 300);
        myImage.setColor(new Color(130, 130, 130));
        
        int width = 10;
        int halfwidth = (int)(width/2.0);
        myImage.fillRect(stack1.getXLoc()-halfwidth, 115, width, 235);
        myImage.fillRect(stack2.getXLoc()-halfwidth, 115, width, 235);
        myImage.fillRect(stack3.getXLoc()-halfwidth, 115, width, 235);
        
        setBackground(myImage);
        addObject(stepCounter, 100, 64);
        makeInstructions();
        makeColors();
        makeRings(numrings);
        addRings();
    }
    public void addRings()
    {
        for(int a = 0; a < rings.size(); a ++) {
            Ring b = rings.get(a);
            stack1.addRing(b);
            addObject(b, stack1.getXLoc(), 360 - (20 * stack1.getRings()));
        }
    }
    public void makeRings(int ringsToAdd)
    {
        //All the ring sizes between 125 and 75
        double increment = (75.0/(ringsToAdd-1));
        for(int a = 0; a < ringsToAdd; a++) {
            rings.add(new Ring(colors.get(a), 125 - (int)(a * increment)));
        }
    }
    public void makeColors()
    {
        for(int a = 0; a < 20; a++) {
            colors.add(new Color(Greenfoot.getRandomNumber(150), Greenfoot.getRandomNumber(200), Greenfoot.getRandomNumber(200)));
        }
    }
    public void doInstructionSteps()
    {
        if(instructionSubStep == 0) {
            if(!moving.moving())
                instructionSubStep = oinstructionSubStep + 1;
        }
        else if(instructionSubStep == 1) {
            //Glide to (CurrentX, 100)
            if(icurrentInstruction.getFrom() == 1) {
                moving = stack1.getTopRing();
                stack1.removeTopRing();
            }
            else if(icurrentInstruction.getFrom() == 2) {
                moving = stack2.getTopRing();
                stack2.removeTopRing();
            }
            else {
                moving = stack3.getTopRing();
                stack3.removeTopRing();
            }
            if(moving != null) {
                moving.glideTo(moving.getX(), 100, 30);
            }
            oinstructionSubStep = instructionSubStep;
            instructionSubStep = 0;
        }
        else if(instructionSubStep == 2) {
            int destinationX = 0;
            if(icurrentInstruction.getTo() == 1) {
                destinationX = stack1.getXLoc();
            }
            else if(icurrentInstruction.getTo() == 2) {
                destinationX = stack2.getXLoc();
            }
            else {
                destinationX = stack3.getXLoc();
            }
            moving.glideTo(destinationX, moving.getY(), 20);
            oinstructionSubStep = instructionSubStep;
            instructionSubStep = 0;
        }
        else if(instructionSubStep == 3) {
            int destinationY = 0;
            if(icurrentInstruction.getTo() == 1) {
                destinationY = 340 - (20 * (stack1.getRings()));
            }
            else if(icurrentInstruction.getTo() == 2) {
                destinationY = 340 - (20 * (stack2.getRings()));
            }
            else {
                destinationY = 340 - (20 * (stack3.getRings()));
            }
            moving.glideTo(moving.getX(), destinationY, 30);
            oinstructionSubStep = instructionSubStep;
            instructionSubStep = 0;
        }
        else if(instructionSubStep == 4) {
            if(icurrentInstruction.getTo() == 1) {
                stack1.addRing(moving);
            }
            else if(icurrentInstruction.getTo() == 2) {
                stack2.addRing(moving);
            }
            else {
                stack3.addRing(moving);
            }
            incrementCounter();
            finishedInstruction = true;
        }
    }
    public void incrementCounter()
    {
        stepCounter.add(1);
    }
    public void act()
    {
        if(!(stack3.getRings() == numrings)) {
            if(finishedInstruction)
            {
                currentInstruction += 1;
                oinstructionSubStep = 1;
                instructionSubStep = 1;
                icurrentInstruction = instructions.get(currentInstruction-1);
                finishedInstruction = false;
            }
            doInstructionSteps();
        }
    }
    public Instruction getInstruction(int step)
    {
        return instructions.get(step);
    }
    public void clearOutput()
    {
        for(int a = 0; a < 20; a++) {
            System.out.println(" ");
        }
    }
    public void makeInstructions()
    {
        HTower(numrings, start, finish);
    }
    public void HTower(int numRings, int current, int destination)
    {
        int spare = 0;
        if(current == 1) {
            if(destination == 2) {
                spare = 3;
            }
            else {
                spare = 2;
            }
        }
        else if(current == 2) {
            if(destination == 1) {
                spare = 3;
            }
            else {
                spare = 1;
            }
        }
        else {
            if(destination == 2) {
                spare = 1;
            }
            else {
                spare = 2;
            }
        }
        Instruction returnInstruction = null;
        
        if(numRings != 1) {
            HTower(numRings -1, current, spare);
            instructions.add(new Instruction(current, destination));
            HTower(numRings -1, spare, destination);
        }
        else {
            instructions.add(new Instruction(current, destination));
        }
    }
}
